# Copyright(c) 1998, Space Science and Engineering Center, UW-Madison
# Refer to "McIDAS Software Acquisition and Distribution Policies"
# in the file  mcidas/data/license.txt

# $Id: LoopManagerlite.tcl,v 1.19 2001/10/18 17:00:14 tsmith Exp $

#==========================================================================#
# LoopManager -- this module is used to create/load and manage loops       #
#==========================================================================#
# Global Structures/Arrays
#	LMLoops() -- Allow up to 99 loops (from DEFLOOP) and arrange as follows
#		  -- "1,N" -- Holds the NAME of the loop
#		  -- "1,F" -- Holds the number of FRAMES in the loop
#		  -- "1,O" -- List of the order of the frames in the loop 
#
#	LMFrames() -- Allow up to 500 frames
#		   -- "1,S" -- Status Line for each frame -- same as in the ListBox
#                           -- Position(0-3) Frame(5-7) Sensor(9-20) Band(22-25) Date(27-38) Time(40-47) DataSource(49-end)
#		   -- "1,P" -- if Status is Panelled then store Row and Column info here
#			     as follows "CC--RR" where CC is column # and RR is row #
#		   -- "1,M" -- Marker value - 0 none any other is it's loop position
#		   -- "1,F" -- Flags "I " Image " G" Grid -- can have both "IG" is IMAGE and GRID	
#		   -- "1,C" -- Color Flags (1 is on 0 is off) by bit so up to 32 levels
#
#	LMPanels() -- size specified by user but each must be 32*32 pixels so
#		    Framewidth/32 = MAX_PANEL_WIDE and FrameWidth/32 = MAX_PANEL_HIGH
#		   -- since panelling is less common -- define each as we go
#		   -- "FRAME_NUMBER-COLUMN-ROW" -- Status (I)mage (G)rid (E)mpty
#
#	LMLoopCount   -- Number of defined loops 
#	LMFrameCount  -- Number of frames allowed (UCPeek 13)
#	LMInitialized -- Has the loop manager been initialized 
#	LMVisible     -- Is the LoopManager visible or not
#	LMCurrentLoop -- loop that is selected for viewing or editing
#       LMLoopNameVar -- used to edit the loops name
#	LMActiveFrame -- Active McIDAS frame nubmer
#
#	-- following variables used to handle the Frame list box selections
#	LMFramesSelected -- how many frames are currently selected
#	LMButton1Press   -- index of Button1Press
#
#       -- List Box handles
#	LMListLoops	-- List box of loop names
#	LMListFrames	-- List box of frames
#--------------------------------------------------------------------------#
#
#	LMInit -- Initializes the loop manager
#	LMView -- displays the loop manager window
#	LMRefreshFrames -- refreshes the frame data
#	LMRefreshLoops -- refreshes the loop data
#	LMDisplayFrames -- places frames in the view window
#	LMNewLoop -- preps for definition of a new loop
#	LMButton1Press -- trigured when button 1 pressed inside the frame list
#	LMButton1Release -- triggered when button 1 released inside the frame list
#	LMClearFlags -- used to clear marked frames inside the frames list
#	LMMarkRange -- used to clear/mark the selected range inside the frames list
#	LMSetLoop -- sets the loop info from the selected frames and new name entered
#	LMDisplayLoops -- displays the loop list in the Loop list box
#	LMLoopEdit -- either Double click or the Edit button selected
#--------------------------------------------------------------------------#
# LMInit -- Sets some global data so that when other routines              #
#           are called the presence of the LoopManager can be              #
#           verified.						           #
#--------------------------------------------------------------------------#
proc LMInit { } \
  {
  global LMInitialized
  global LMLoopCount
  global LMCurrentLoop
  global LMFrames
  global LMFrameCount
  global LMVisible
  global LMActiveFrame LMActiveLoop
  global Vis_or_IR

  
  # -- early exit if already been called
  if { [info exists LMInitialized] } {return }    
  
  # the following variable is now set by the CONTEXT.DAT file  
    #set LMLoopCount 0
  set LMCurrentLoop -1
  set LMActiveLoop -1
  set LMInitialized 1
  set LMFrameCount [UCpeek 13]
  set LMVisible 0
  set LMActiveFrame 1
  # -- Clear all of the MARK position indicators
  for { set i 0 } { $i < $LMFrameCount} { incr i } { set LMFrames($i,M) 0 }

  LMRefreshFrames
  }

#------------------------------------------------------------------------------------------#
# LMRefreshFrames -- reloads all of the frame information (FRMLIST) for each frame	   #
#------------------------------------------------------------------------------------------#
proc LMRefreshFrames { } \
  {
  global LMFrames 		;# the frames array
  global LMFrameCount		;# the number of frames
  global LMVisible
  global LMInitialized
  
  if { ![info exists LMInitialized] } { LMInit}
  #-- will do a FRMLIST command -- if the data is not set for each frame then go ahead
  #-- and initialize it -- else leave it alone
  set LMFrameCount [UCpeek 13]  ;# update in case framew were added
  set FrameList [open "|frmlist.k 1 $LMFrameCount" r] 
  
  #-- burn the first three lines
  gets $FrameList burnit
  gets $FrameList burnit
  gets $FrameList burnit
  
  #-- now each line is a frame info line -- entry 1
  #	# -- No frame information  -- if empty frame
  #	#[#] -- if information is present -- [#] is the number of panels in the frame
  
  for {set i 0} { $i < $LMFrameCount } { incr i } \
    {
    #--- Inititalize the frame data if it hasn't been already
      if { ![info exists LMFrames($i,F)] } \
        {
        set LMFrames($i,S) ""
	set LMFrames($i,M) 0    ;# no position info set or MARKED
        set LMFrames($i,F) ""   ;# initialize to an empty list
	set LMFrames($i,C) -1   ;# had better be all bits SET
        }
    #--- read in the next line of text
      gets $FrameList line
    #--- get the frame number and weather or not it has panels
      set FrameNumber [lindex $line 0]
      scan $FrameNumber { %[0-9]%[[]%[0-9] } Number burnit Panels
    #--- DataSource or SERVER SubEntry
      set DataSource [lindex [split [string range $line 50 end] "."] 0]  ;# Sub-Entry
    #--- Sensor or SERVER NAME  
      set Sensor [string trim [string range $line 10 21]]		       ;# Server Name
    #--- if the frame is marked -- set to a position -- save that else display as "---"
      if { !$LMFrames($i,M) } { set Position "---" } else { set Position $LMFrames($i,M) }
    #--- if there is frame information  
    if { [string first "No frame information" $line] ==-1} \
      {
      set LMFrames($i,S) [format "%4s %3s %12s %4s %12s %8s %s" $Position $Number $Sensor \
    			[string range $line 23 26] [string range $line 28 39] [string range $line 41 48] \
			$DataSource]
      set LMFrames($i,F) "I" 
      }\
    else\
      { 
      set LMFrames($i,S) [format "%4s %3s No frame information" "---" $Number]
      set LMFrames($i,F) "" 
      }  			
    }
  
  #-- make sure eat all of the data in the pipe
  while { [gets $FrameList line] > -1 } {}  
  catch { close $FrameList }
  }

#--------------------------------------------------------------------------#
# LMView -- forces the LMWindow to appear if it doesn't already            #
#                  exist.  						   #
#--------------------------------------------------------------------------#
proc LMView { } \
  {
  global LMInitialized
  global LMLoopCount
  global LMFrames
  global LMVisible
  global LMLoopNameVar
  global LMCurrentLoop
  global LMLoopList LMFrameList  
    
  # -- Make sure we're initialized
  if { ![info exists LMInitialized] } { LMInit }
  # -- If already visible just return
  if { $LMVisible } { wm deiconify .tlLM; focus .tlLM; return }

  # -- now create the window
  toplevel .tlLM 
  wm title .tlLM "Loop Manager"
    #--- Main frame
    frame .tlLM.fmMain -relief raised -borderwidth 3
      # -- The Loops frame
      frame .tlLM.fmMain.fmLoops 
        # -- The ListBox frame
        frame .tlLM.fmMain.fmLoops.fmList 
          set LMLoopList [MakeListBox .tlLM.fmMain.fmLoops.fmList  [format "%-30s %s" "Loop List" "# Frames"] ""]
        pack .tlLM.fmMain.fmLoops.fmList -side top -padx 5 -pady 5 -expand true -fill both
        # -- Editing options frame
        frame .tlLM.fmMain.fmLoops.fmLoopOptions -relief flat
          button .tlLM.fmMain.fmLoops.fmLoopOptions.btNew -text "New Loop" -width 12 -command LMNewLoop
          button .tlLM.fmMain.fmLoops.fmLoopOptions.btDelete -text "Delete" -width 12 -command LMDeleteLoop
          pack .tlLM.fmMain.fmLoops.fmLoopOptions.btNew -side left -pady 5 -padx 10
          pack .tlLM.fmMain.fmLoops.fmLoopOptions.btDelete -side left -pady 5 -padx 10
        pack .tlLM.fmMain.fmLoops.fmLoopOptions -side top -padx 5 -pady 5
      pack .tlLM.fmMain.fmLoops -side left -fill both -expand false -padx 5 -pady 5
    pack .tlLM.fmMain -side top  
    #-- Exit frame
    frame  .tlLM.fmControls -relief raised -borderwidth 3
      button .tlLM.fmControls.btDismiss -text "Dismiss" -width 12 \
        -command {
	   SaveContext  ; # may need to alter to just save LOOPS ??
	   destroy .tlLM 
	   }
      pack .tlLM.fmControls.btDismiss -side top -pady 10
    pack .tlLM.fmControls -side top -fill both -expand true  


  #--- configure the LOOP list box
    bind $LMLoopList <Double-1> {   set line [lindex [$LMLoopList curselection] 0]
                                    set LMCurrentLoop $line 
                                    set LMLoopNameVar $LMLoops($LMCurrentLoop,N)
                                    LMLoopEdit }

  #-- make sure to reset the flag when the window closes as well 
    bind .tlLM <Destroy> { set LMVisible 0 }

  # -- flag so that knows dialog is visible
    set LMVisible 1
    set LMButton1Pressed 0
  
  #-- initialize everything
    LMRefreshFrames
    LMDisplayLoops
    set LMLoopNameVar ""
    set LMCurrentLoop 0
  }
  
#----------------------------------------------------------------------------------#
# LMLoopEdit ( YLoc ) -- Begin Loop editing                                        #
#----------------------------------------------------------------------------------#
proc LMLoopEdit { } \
  {
  global LMLoopCount
  global LMLoopNameVar
  global LMCurrentLoop  
  global LMFramesSelected
  global LMButton1Pressed
  global LMLoops 
  global LMFrameList
  global LMSaveIt

  if { $LMLoops($LMCurrentLoop,F)<0 } return

  toplevel .tlLoopEdit
  wm title .tlLoopEdit "Edit Loop"
  
  #-- Main Frame
  frame .tlLoopEdit.fmMain -relief raised -borderwidth 3
    #-- Now the frames list for forming the loop sequence
    frame .tlLoopEdit.fmMain.fmFrames 
      #--- List box frame
      frame .tlLoopEdit.fmMain.fmFrames.fmList 
        set LMFrameList [MakeListBox .tlLoopEdit.fmMain.fmFrames.fmList "Frame List" \
                            [format "%4s %3s %-12s %4s %12s %8s %s  " "" "#" "Sensor" "Band" "Date    " "Time  " "Data Source"]]
        pack .tlLoopEdit.fmMain.fmFrames.fmList		       
      # -- SET LOOP frame etc..
      set LMSaveIt 0  ;# initialize flag to NO SAVE
      frame .tlLoopEdit.fmMain.fmFrames.fmSetit 
        label .tlLoopEdit.fmMain.fmFrames.fmSetit.lbName -text "Loop Name:" -anchor e 
        entry .tlLoopEdit.fmMain.fmFrames.fmSetit.enName -width 30 -textvariable LMLoopNameVar -background white
        button .tlLoopEdit.fmMain.fmFrames.fmSetit.btSetit -text "Set Loop" -command {set LMSaveIt 1; LMSetLoop; destroy .tlLoopEdit} 
        pack .tlLoopEdit.fmMain.fmFrames.fmSetit.lbName -side left
        pack .tlLoopEdit.fmMain.fmFrames.fmSetit.enName -side left -fill x -expand true
        pack .tlLoopEdit.fmMain.fmFrames.fmSetit.btSetit -side right
        pack .tlLoopEdit.fmMain.fmFrames.fmSetit -side top -pady 5 -fill x
      pack .tlLoopEdit.fmMain.fmFrames.fmSetit -side top -padx 5 -pady 10 -expand true -fill both
    pack .tlLoopEdit.fmMain.fmFrames -side right -fill both -expand false -padx 5 -pady 5
  pack .tlLoopEdit.fmMain -side top
  #-- Exit frame
  frame  .tlLoopEdit.fmControls -relief raised -borderwidth 3
    button .tlLoopEdit.fmControls.btDismiss -text "Dismiss" -width 12 -command {destroy .tlLoopEdit }
    pack .tlLoopEdit.fmControls.btDismiss -side top -pady 10
  pack .tlLoopEdit.fmControls -side top -fill both -expand true  
    
  #--- configure the FRAME List Box
    $LMFrameList config -selectmode extended
    bind $LMFrameList <Button-1> { LMButton1Press %W %y NONE }
    bind $LMFrameList <ButtonRelease-1> { LMButton1Release %W %y NONE }
    bind $LMFrameList <Control-Button-1> { LMButton1Press %W %y CONTROL }
    bind $LMFrameList <Control-ButtonRelease-1> { LMButton1Release %W %y CONTROL }
    bind $LMFrameList <Shift-Button-1> { LMButton1Press %W %y SHIFT }
    bind $LMFrameList <Shift-ButtonRelease-1> { LMButton1Release %W %y SHIFT }

  set LMButton1Pressed 0
  LMDisplayFrames

  # -- set focus to entry widget -- highlight the selection as well
    focus .tlLoopEdit.fmMain.fmFrames.fmSetit.enName 
    .tlLoopEdit.fmMain.fmFrames.fmSetit.enName icursor end
    .tlLoopEdit.fmMain.fmFrames.fmSetit.enName select range 0 end
    
  tkwait window .tlLoopEdit

  #-- remove the loop if none marked and is the latest to be added
  if { $LMCurrentLoop == [expr $LMLoopCount-1] && !$LMSaveIt && ![llength $LMLoops($LMCurrentLoop,O)] } \
    {
    unset LMLoops($LMCurrentLoop,N)
    unset LMLoops($LMCurrentLoop,F)
    unset LMLoops($LMCurrentLoop,O) 
    incr LMCurrentLoop -1
    incr LMLoopCount -1
  
    LMDisplayLoops
    LMUpdateLoopButtons 
    return;
    }	  
  LMDisplayLoops
  }
  
#-----------------------------------------------------------------------------------------#
# LMNewLoop -- starts a new loop for editing  						  #
#-----------------------------------------------------------------------------------------#
proc LMNewLoop { } \
  {
  global LMLoopCount
  global LMLoopNameVar
  global LMLoops
  global LMCurrentLoop
  
  set LMCurrentLoop $LMLoopCount
  incr LMLoopCount
  set LMLoopNameVar "Loop$LMLoopCount"
 
  # -- set up the loop header stuff
  set LMLoops($LMCurrentLoop,N) $LMLoopNameVar
  set LMLoops($LMCurrentLoop,F) 0
  set LMLoops($LMCurrentLoop,O) ""
  
  LMLoopEdit
  }
  
#----------------------------------------------------------------------------#
# LMSetLoop() -- takes the current loop name and selected frames and builds  #
#		a loop from them					     #
#----------------------------------------------------------------------------#
proc LMSetLoop { } \
  {
  global LMLoopCount
  global LMFrameCount
  global LMCurrentLoop
  global LMLoopNameVar  
  global LMFramesSelected
  # -- arrays 
  global LMLoops
  global LMFrames     
 
  # -- set up the loop header stuff
  set LMLoops($LMCurrentLoop,N) $LMLoopNameVar
  set LMLoops($LMCurrentLoop,F) $LMFramesSelected
  set LMLoops($LMCurrentLoop,O) ""

  set find 1
  # -- now build the list of frames in order
  while { $LMFramesSelected } \
    {
    # -- Loop through until all of the entries are pulled off in order
    for { set i 0} { $i<$LMFrameCount } { incr i } \
      {
      if { $LMFrames($i,M) == $find} \
        {
	set LMFrames($i,F) $LMCurrentLoop
        append LMLoops($LMCurrentLoop,O) " $i"
	incr find
        incr LMFramesSelected -1
  	}
      }
    }
  LMUpdateLoopButtons 
  
  #----Desired to add functionality to update the SHOW buttons and make this loop active
    LMSetFrameAndLoop [expr [lindex $LMLoops($LMCurrentLoop,O) 0] + 1 ] $LMCurrentLoop
  }
#------------------------------------------------------------------------------------------#
# LMDisplayFrames -- fills the Frames list box with the frame information -- if a loop     #
#		is active, then those  frames are hilighted                                #
#------------------------------------------------------------------------------------------#
proc LMDisplayFrames { } \
  {
  global LMLoops
  global LMFrames
  global LMFrameCount
  global LMInitialized
  global LMCurrentLoop
  global LMFramesSelected
  global LMLoopList LMFrameList

  if { ![info exists LMInitialized] } \
    {
    LMInit
    LMView	;# LMView will call this routine back
    return
    }
    
  set LMFramesSelected 0
  #-- reset the list based on the current loop
  if { $LMCurrentLoop>=0 }\
    {
    LMClearFlags 
    set position 1
    foreach value $LMLoops($LMCurrentLoop,O) \
      {
      set LMFrames($value,M) $position
      set LMFrames($value,S) [format "%4s%s" $position [string range $LMFrames($value,S) 4 end]]
      incr position
      incr LMFramesSelected
      }
    }

  #-- Clear the ListBox first  
  $LMFrameList delete 0 end
  for { set i 0 } { $i < $LMFrameCount } { incr i } \
    {
    $LMFrameList insert $i $LMFrames($i,S) 
    # -- add highlighting here
    if { $LMFrames($i,M) } { $LMFrameList selection set $i } 
    }
  }
#-----------------------------------------------------------------------------------------#
# LMButton1Press { LBox YLoc Flags } -- button 1 press was detected so handle it here     #
#		-- LBox  -- the list box handle						  #
#		-- YLoc  -- Y location of the cursor inside the listbox			  #
#		-- Flags -- CONTROL -- the control key was held down		  	  #
#			 -- SHIFT   -- the shift key was held down			  #
#			 -- NONE    -- no keys held down				  #
#-----------------------------------------------------------------------------------------#
proc LMButton1Press { LBox YLoc Flags } \
  {
  global LMButton1Pressed
  global LMFrames

  set restoreposition [$LBox index @0,0]
  set line [$LBox index @0,$YLoc]

  if { [string match $Flags NONE] } \
    {
    # -- No modifier keys held down
    # -- clear all markers
    LMClearFlags 
    # -- set starting index
    set LMButton1Pressed $line
    } \
  elseif { [string match $Flags SHIFT] } \
    {
    # -- clear all markers
    LMClearFlags 
    # -- if no previouse press then set
    if { ! $LMButton1Pressed } { set LMButton1Pressed $line }
    # -- reset first item selected
    LMMarkRange $LBox $LMButton1Pressed $line 0
    }\
  elseif { [string match $Flags CONTROL] } \
    {
    # -- don't clear the flags until the entire section is pulled for clearing
    set LMButton1Pressed $line 
    }  

  $LBox yview $restoreposition  
    
  }
#-----------------------------------------------------------------------------------------#
# LMButton1Release { LBox YLoc Flags } -- detects the release of mouse button 1  	  #
#		-- LBox  -- the list box handle						  #
#		-- YLoc  -- Y location of the cursor inside the listbox			  #
#		-- Flags -- CONTROL -- the control key was held down		  	  #
#			 -- SHIFT   -- the shift key was held down			  #
#			 -- NONE    -- no keys held down				  #
#-----------------------------------------------------------------------------------------#
proc LMButton1Release { LBox YLoc Flags } \
  {
  global LMButton1Pressed
  global LMFramesSelected
 
  #-- save the location
  set line [$LBox index @0,0]
  set LMButton1Released [$LBox index @0,$YLoc] 
  if { [string match $Flags NONE] } \
    {
    LMMarkRange $LBox $LMButton1Pressed $LMButton1Released 0
    } \
  elseif { [string match $Flags SHIFT] } \
    {
    LMClearFlags 
    LMMarkRange $LBox $LMButton1Pressed $LMButton1Released 0
    }\
  elseif { [string match $Flags CONTROL] } \
    {
    LMMarkRange $LBox $LMButton1Pressed $LMButton1Released $LMFramesSelected 
    }   
  # -- and reset location   
  $LBox yview $line  
  }
#-----------------------------------------------------------------------------------------#  
# LMClearFlags -- clears all of the marked lines for the list box		          #
#	       -- removes Position number 		                                  #
#-----------------------------------------------------------------------------------------#
proc LMClearFlags { } \
  {
  global LMFrameCount
  global LMFrames
  global LMFramesSelected
  global LMFrameList
  
  for { set i 0 } { $i < $LMFrameCount } {incr i} \
    {
    if { $LMFrames($i,M) } \
      {
      # -- need to remove the second lindex value and replace it with ---
      set LMFrames($i,S) [format " ---%s" [string range $LMFrames($i,S) 4 end]]
      set LMFrames($i,M) 0
      $LMFrameList delete $i
      $LMFrameList insert $i $LMFrames($i,S)
      }
    }
  set LMFramesSelected 0
  }
#------------------------------------------------------------------------------------------#
# LMMarkRange { LBox StartIndex EndIndex StartNumber }  -- sets each lines position nubmer #
#		-- LBox -- handle of the list box 					   #
#		-- StartIndex  -- set this line to StartNumber				   #
#		-- EndIndex    -- set this line to StartNumber + diff(StartIndex,EndIndex) #
#		-- StartNumber -- begin lableing with this value			   #
#------------------------------------------------------------------------------------------#
proc LMMarkRange { LBox StartIndex EndIndex StartNumber } \
  {
  global LMFrames    
  global LMFramesSelected
  global LMFrameCount

  # -- Redraw array declared on the fly -- if a marking changes then this is set so that
  # -- -1 indicates it needs to be redrawn
  # --  1 indicates that it needs to be assigned a Position number
  # -- the list box entry can be redrawn
  # -- number of entries to mark
  set Marked 0
  # -- need a first pass to remove all flags that are already set in the range between the indexes
  if { $StartIndex > $EndIndex} \
    {
    set start $EndIndex
    set end $StartIndex
    }\
  else \
    {
    set start $StartIndex
    set end $EndIndex
    }  
  for { set i $start } { $i <= $end} { incr i }\
    {
    if { $LMFrames($i,M) } \
      {
      # -- was already marked so need to CLEAR this one first
      set compTo $LMFrames($i,M)
      set LMFrames($i,M) 0
      set Redraw($i) -1
      incr LMFramesSelected -1
      incr StartNumber -1
      for { set j 0 } {$j < $LMFrameCount } { incr j } \
        {
	# -- Loop through all of them dec. the -M variable if greater than compTo
	if { $LMFrames($j,M) > $compTo }\
	  {
	  incr LMFrames($j,M) -1
	  set Redraw($j) -1
	  }
	}
      }\
    else\
      {
      # -- need to mark this one -- so set flag indicating needs to be marked
      set Redraw($i) 1
      incr Marked
      incr LMFramesSelected
      }  
    }  

  # -- loop through the range again check the Redraw flag: if 1 then set correctly
  if { $StartIndex > $EndIndex } { set Index [expr $Marked -1]} else { set Index 0 }
  for { set i $start } { $i <= $end} { incr i }\
    {
    if { [info exists Redraw($i)] } \
      {
      if { $Redraw($i) == 1 } \
        {
        set LMFrames($i,M) [expr $StartNumber + $Index +1] 
        if { $StartIndex > $EndIndex } { incr Index -1} else { incr Index }
	}
      }
    }
  for { set i 0 } { $i<$LMFrameCount } { incr i } \
    {
    if { [info exists Redraw($i)] }\
      {
      if { $LMFrames($i,M) } \
        {
        set LMFrames($i,S) [format "%4s%s" $LMFrames($i,M) [string range $LMFrames($i,S) 4 end]]
        $LBox delete $i 
        $LBox insert $i $LMFrames($i,S)
	$LBox selection set $i
	}\
      else\
        {
        set LMFrames($i,S) [format "%4s%s" " ---" [string range $LMFrames($i,S) 4 end]]
        $LBox delete $i 
        $LBox insert $i $LMFrames($i,S)
	}
      }	
    }  
  }

#----------------------------------------------------------------------------------#
# LMDisplayLoops -- displays the current loop list				   #
#----------------------------------------------------------------------------------#
proc LMDisplayLoops { } \
  {
  global LMFrames
  global LMLoopCount
  global LMInitialized
  global LMLoops
  global LMLoopList LMFrameList

  if { ![info exists LMInitialized] } \
    {
    LMInit
    LMView	;# LMView will call this routine back
    return
    }
  #-- Clear the ListBox first  
  $LMLoopList delete 0 end
  for { set i 0 } { $i < $LMLoopCount } { incr i } \
    {
    if { $LMLoops($i,F) > 0 } \
      {
      $LMLoopList insert $i [format "%-30s %s" $LMLoops($i,N) $LMLoops($i,F)]
      } \
    else \
      {
      $LMLoopList insert $i [format "%-30s %s" $LMLoops($i,N) "Inactive"]
      }  
    }
  }
#---------------------------------------------------------------------------------#
# LMDeleteLoop -- Deletes all of the selected loops				  #
#---------------------------------------------------------------------------------#
proc LMDeleteLoop { } \
  {
  global LMLoopCount
  global LMLoopNameVar
  global LMCurrentLoop
  global LMLoops
  global LMLoopList
  global LMActiveLoop
  
  set deleted 0
  set ResetLoopSettings 0
  # -- wipe out the loops in memory  
  foreach value [$LMLoopList curselection] \
    {
    #---set so that these frames no longer belong to a loop
    unset LMLoops($value,N)
    unset LMLoops($value,O)
    unset LMLoops($value,F)
    # -- increment the count of deleted terms
    incr deleted
    if { $value == $LMActiveLoop } \
      {
      set ResetLoopSettings 1
      }
    }
  set i 0
  #--restructure the lists
  while { $i < $LMLoopCount } \
    {
    # while haven't checked the entire space keep looping
    if { ![info exists LMLoops($i,N)] } \
      {
      # as soon as an opening appears start save the position and
      # start looking for a filled slot
      set ToFill $i
      while { $i < $LMLoopCount && ![info exists LMLoops($i,N)]} { incr i }
      # now have to make sure that we found an item
      if { $i < $LMLoopCount }\
        {
	# -- copy the data back to "New"
	set LMLoops($ToFill,N) $LMLoops($i,N)
	set LMLoops($ToFill,F) $LMLoops($i,F)
	set LMLoops($ToFill,O) $LMLoops($i,O)
	# -- remove the "Old" slot
	unset LMLoops($i,N)
	unset LMLoops($i,F)
	unset LMLoops($i,O)
	# -- reposition i
	set i $ToFill
	}
      }
    incr i  
    }
  set LMLoopCount [expr $LMLoopCount - $deleted]  
  set LMCurrentLoop -1
  LMDisplayLoops
  LMUpdateLoopButtons 
  
  set LMLoopNameVar ""
  if { $ResetLoopSettings || $LMActiveLoop == -1 } \
    {
    LMSetFrameAndLoop 1 -2
    }
  }
#---------------------------------------------------------------------------#
# LMUpdateLoopbuttons{} -- used to look for and update all of the FrameLoop #
#                          buttons that may need updating after an image is #
#                          loaded					    #
#---------------------------------------------------------------------------#
proc LMUpdateLoopButtons { } \
  {
  global LMLoopButtonList

  foreach i $LMLoopButtonList \
    {
    LMSetLoopButton $i.fmLoopButton.mbLoop
    }
  }

#---------------------------------------------------------------------------#
# LMCreateLoopButton {} -- creates a loop button that is managed from here  #
#		-- w -- parent to create the loop button inside of          #
#---------------------------------------------------------------------------#
proc LMCreateLoopButton { w label IncludeEntry } \
  {
  global LMActiveFrame LMActiveLoop
  global LMLoopButtonList

  #--- make sure that the LoopManager is initialized  
  LMInit 

  #-- Initialize the starting frame
  frame $w.fmLoopButton
    menubutton $w.fmLoopButton.mbLoop \
	-text $label \
	-menu $w.fmLoopButton.mbLoop.menu \
	-relief raised \
	-borderwidth 2 \
	-indicatoron 1 \
	-direction above
    pack $w.fmLoopButton.mbLoop -side left 

    if { ![string match $IncludeEntry "NO"] } \
      {
#ml      label $w.fmLoopButton.lbLoop -text "Frame:" 
      entry $w.fmLoopButton.enLoop \
	-textvariable LMActiveFrame \
	-state disabled \
	-width 4 
#ml      pack $w.fmLoopButton.lbLoop -side left    
      pack $w.fmLoopButton.enLoop -side left -fill x -expand true
      }
  pack $w.fmLoopButton -side left
  
  # -- And Now rebuild the entire thing from scratch
  LMSetLoopButton $w.fmLoopButton.mbLoop
  bind $w <Destroy> "LMRemoveLoopButton  $w"
  lappend LMLoopButtonList $w

  return $w.fmLoopButton.mbLoop   ;# let calling routine have ID of the button
  }


#---------------------------------------------------------------------------#
# LMSetLoopButton {} -- sets/resets the loop button passed in               #
#---------------------------------------------------------------------------#
proc LMSetLoopButton { w } \
  {
  # -- these variables defined inside the LOOPMANAGER
  global LMLoops LMLoopCount
  global LMFrames LMFrameCount
  global LMActiveFrame LMActiveLoop
  global added

 
  set added 0

  catch { destroy $w.menu }
    
  set m [menu $w.menu -tearoff 0]
  for { set i 0 } { $i < $LMLoopCount} { incr i } \
    {
    # make sure the loop is valid first -- otherwise just skip it
    if { $LMLoops($i,F) > 0 } \
      {
     $m add cascade -label $LMLoops($i,N) -menu $m.sub$i 
      incr added
      set added2 0
      set m2 [menu $m.sub$i -tearoff 0]
     foreach value $LMLoops($i,O) \
       {
        # --- break apart the label into what we really need only
        if { $added2 > 30 } \
          {
	  set added2 0
  	  $m2 add cascade -label "MORE" -menu $m2.sub
	  set m2 [menu $m2.sub -tearoff 0]
	  }
        set Marked($value) 1
       if { [string first "No frame information" $LMFrames($value,S)] ==-1} \
          {
          set line [string range $LMFrames($value,S) 27 47]
           set line [format "%s %12s" [expr $value+1] $line]
          }\
       else { set line [format "%3s %s" [expr $value+1] "No frame information"] }  			
#ml       $m2 add command -label $line -command "LMSetFrameAndLoop [expr $value+1] $i"
	$m2 add command -label $line -command "setImageFrame [expr $value+1]"
        incr added2
	}
      } 
    }
  #--Now catch all of the unmarked frames -- those not in a loop and set
  set columnBreak no
  for { set i 0} { $i < $LMFrameCount } {incr i} \
    {
    if { ![info exists Marked($i)] } \
      {

      if { $added > 30 } \
        {
	set added 0
	$m add cascade -label "MORE" -menu $m.sub
	set m [menu $m.sub -tearoff 0]

	}
      if {$i < 9} {set space " "} else {set space ""}
      set hyphon "-"
      set position [string range $LMFrames($i,S) 5 7]
      set position [string trim $position " "]
      if { [string first "No frame information" $LMFrames($i,S)] == -1} \
        {
	set line [string range $LMFrames($i,S) 27 47]
	set line "$space$position$hyphon$line"
        set line [format "%12s" $line]
        }\
      else { set line "$space[expr $i+1]-No frame information"}

#ml      $m add command -label $line -command "LMSetFrameAndLoop [expr $i +1] -1"
      $m add command -label $line -command "setImageFrame [expr $i +1]"
      incr added
      set columnBreak no
      }
    }
  # -- Now build an ALL FRAMES entry  
#ml    $m add cascade -label "ALL FRAMES" -menu $m.subALL 
#ml    set m2 [menu $m.subALL -tearoff 0]
#ml    for { set i 0} { $i < $LMFrameCount } {incr i} \
#ml     {
#ml      if { [string first "No frame information" $LMFrames($i,S)] ==-1} \
#ml        {
#ml        set line [string range $LMFrames($i,S) 27 47]
#ml        set line [format "%s %12s" [expr $i+1] $line]
#ml        }\
#ml      else { set line [format "%3s %s" [expr $i+1] "No frame information"] }  			
#ml      $m2 add command -label $line -command "LMSetFrameAndLoop [expr $i +1] -3"
#ml      }
  }

#-----------------------------------------------------------------------#
# LMAddDeleteFrame {} -- Added for McIDAS-Lite.  The "Show Frames"
#			 menu has been changed such that each item
#			 in the menu allows the User to include or
#			 exclude a particular frame.  It no longer
#			 is used to display a particular frame, this
#			 can now only be done using the left/right
#			 arrow buttons.
#-----------------------------------------------------------------------#
proc LMAddDeleteFrame {} \
{
   global frameCheckButton
   global LMFrameCount
   global loop_button_counter


   # Prevent execution of the 'LS' command during GUI startup
   if {![info exists loop_button_counter]} \
   {
	set loop_button_counter 1
   } \
   else \
   {
	incr loop_button_counter
   }

   if {[expr {$loop_button_counter < $LMFrameCount}]} {return}

   # Create a loop sequence consisting of all frames for which the
   # corresponding checkbox is checked.
   set loopSequence [list]

   for {set i 1} {$i <= [UCpeek 13]} {incr i} \
   {
	if {[string compare $frameCheckButton($i) "YES"] == 0} \
	{
	   lappend loopSequence $i
	}
   }

   # Set new loop sequence
   mcidas exec "LS $loopSequence"
}



#-----------------------------------------------------------------------#
# LMSetFrameAndLoop { Frame Loop } -- sets to correct loop and to correct
# 	frame inside of the loop
#-----------------------------------------------------------------------#
proc LMSetFrameAndLoop { frameNum loopNum } \
  {
  global LMLoops 
  global LMActiveFrame
  global LMActiveLoop
  global LMLoopCount
  global LMFrameCount

  set LMActiveFrame $frameNum
  # -- chance on startup of getting a frame number of 0 -- illegal so increment to 1
  if { !$frameNum } \
    {
    incr LMActiveFrame 1
    }

  #ml -4 means to set the frame and return
  if {$loopNum == -4} \
  {
	setImageFrame $frameNum
	return
  }

  # -- if the loop selected has a negative length -- don't set but exit
  if { $loopNum>=0 && $LMLoops($loopNum,F) < 0 } return
  # -- -3 means to just set the absolute frame number and return
  if { $loopNum==-3} \
    {
    # -- put all of the frames in a list
    set doit ""
    for { set i 0} { $i < $LMFrameCount } {incr i} \
      {
      set doit "$doit [expr $i+1]" 
      }
    mcidas exec "LS $doit" 
    mcidas exec "SF $LMActiveFrame" ;#frameNum"
#ml    setLoopPosScaleLen
    set LMActiveLoop -3
    return
    }

  if { $loopNum != $LMActiveLoop } \
    {
    set LMActiveLoop $loopNum
  
    #-- now build the loop if valid
    set doit ""
    if { $loopNum>-1 } \
      {
      foreach i $LMLoops($loopNum,O) { set doit "$doit [expr $i+1]" }
      } \
    else \
      {
      # --- -1 or -2 indicates a loop reset
      # --- make sure that a value if -2 is not used 
      set LMActiveLoop -1
      # --- get all of the existing frames in a loop
      catch { unset Marked }
      for { set i 0 } { $i < $LMLoopCount} { incr i } \
        {
	if { $LMLoops($i,F) > 0 } \
	  {
          foreach value $LMLoops($i,O) { set Marked($value) 1 }
	  }
        } 
      #--Now catch all of the unmarked frames -- those not in a loop and set
      for { set i 0} { $i < $LMFrameCount } {incr i} \
        {
        if { ![info exists Marked($i)] } { set doit "$doit [expr $i+1]" }
        }
      }
    #--- update the framenum
      set frameNum [lindex $doit 0 ]	
    
    mcidas exec "LS $doit" 
    }
  #-- set to the correct frame
  mcidas exec "SF $LMActiveFrame" ;#frameNum
  
  #-- used to update the "VCR" control on the tool bar
#ml  setLoopPosScaleLen

  }

#-----------------------------------------------------------------------#
# LMRemoveLoopButton { } -- used to remove the parent ID from the list  #
#                          of widgets that contain the Loop Buttons     #
#-----------------------------------------------------------------------#
proc LMRemoveLoopButton { w } \
  {
  global LMLoopButtonList
  
  set newlist ""
  
  foreach i $LMLoopButtonList \
    {
    if { ![string match $i $w] } { lappend newlist $i }
    }
  
  set LMLoopButtonList $newlist
  }

#------------------------------------------------------------------------------#
# LoadImageIntoFrame { Group Descriptor Position Band 
#		       Options ImageNumber ShowMap Bar}
#------------------------------------------------------------------------------#
proc LMLoadImageIntoFrame { Group Descriptor Position Band \
			    LoadOptions ImageNumber ShowMap Bar } \
  {
  global LMActiveFrame LMActiveLoop
  global LMFrames LMLoops 
  global LMFrameCount
  global LMLoopCount
  global Vis_or_IR

  
  #---- Build the list of frames to insert elements into
  set LoopList ""
  set StartIndex 0
  
  if { $LMActiveLoop >= 0 } \
    {
    foreach i $LMLoops($LMActiveLoop,O) \
      { 
      set LoopList "$LoopList [expr $i+1]" 
      if { [expr $i+1] == $LMActiveFrame } { set StartIndex $i }
      }
    # -- don't allow loading outside of the loop or doubling back over same frame  
    if { $LMLoops($LMActiveLoop,F) < $ImageNumber } { return }
    } \
  else \
    {
    #--- Need to get all of the frames that are not in a loop and build a list of them
    for { set i 0 } { $i < $LMLoopCount } { incr i } \
      {  
      if { $LMLoops($i,F) > 0 } \
        {
        foreach value $LMLoops($i,O) { set Marked($value) 1 }
	}
      }
    set j 0  
    for { set i 0 } { $i < $LMFrameCount } { incr i} \
      {
      # -- -3 indicates ALL FRAMES are set
      if { ! [info exists Marked($i)] || $LMActiveLoop==-3 } \
        { 
	set LoopList "$LoopList [expr $i + 1]" 
        if { [expr $i+1] == $LMActiveFrame } { set StartIndex $j }
	incr j
	}
      }

    # -- don't allow loading outside of the loop or doubling back over same frame  
    if { [llength $LoopList] < $ImageNumber && $ImageNumber>1 } return  

    }

  # get position of the ActiveFrame in this list
  # Now the frame to set is an offset from there

    set Count [llength $LoopList]    

    if { ! $Count } { set SetFrame $LMActiveFrame } \
    else \
      {
      set Where [expr ($ImageNumber + $StartIndex -1) % $Count]
      set SetFrame [lindex $LoopList $Where]
      }

  mcidas exec "SF $SetFrame"
  
  # Load 'Vis_or_IR' array so that we know if a given frame contains a
  # visible or IR image.  This will be used by the 'Temp' monitor on 
  # the main GUI to decide whether to display data or 'n/a'.
  set current_frame [UCpeek 51]

  # Set UC word to 1 if Vis, or 2 if IR
  if {[string length $current_frame] == 1} \
	{set current_frame 0$current_frame}

  # Determine the appropriate UC word and then set it to
  # 1 by default.  If the image is IR it will be changed
  # to 2.
  set UCWord 19$current_frame
  UCpoke $UCWord 1

  set inputOperator [open "|imglist.k $Group/$Descriptor.$Position FORM=AUX" r]
  while {[gets $inputOperator line] >= 0} \
  {
	if {[string compare [lindex $line 7] "TEMP"] == 0 && \
	    [string compare [lindex $line 5] $Band] == 0} \
	{
	   UCpoke $UCWord 2
	}
  }
  catch {close $inputOperator}

  # If an alias or predifined product from IMGDISP.USER/SITE/USER is used then 
  # don't include the 'group/' in the call to IMGDISP.
  if {[string compare $Group ""] == 0} \
  {
	mcidas exec "IMGDISP $Descriptor.$Position $SetFrame BAND=$Band \
			     $LoadOptions TEXT=''"
  } \
  else \
  {
  	mcidas exec "IMGDISP $Group/$Descriptor.$Position $SetFrame BAND=$Band \
			     $LoadOptions TEXT=''"
  }

  # Include grayscale bar if Bar=YES
  if {[string compare $Bar "YES"] == 0} {mcidas exec "BAR"}

  # Add more 'user friendly' information at the bottom of the image
  mcidas exec "FRMLABEL JUST=L \
	\"  (STYPE) (BAND=(BAND))                         (DAY)  (HHMM) UTC"

  if { $ShowMap } \
    { 
    mcidas exec "EG $SetFrame"
    DrawMap 1 
    }
 
  LMRefreshFrames
  LMUpdateLoopButtons 
  }
  
#------------------------------------------------------------------------------#
# LoadGridIntoFrame { Group Descriptor Position Options GridNumber }
#------------------------------------------------------------------------------#
proc LMLoadGridIntoFrame { Group Descriptor Directory Position LoadOptions GridNumber } \
  {
  global LMActiveFrame LMActiveLoop
  global LMFrames LMLoops 
  global LMFrameCount
  global LMLoopCount
  
  #---- Build the list of frames to insert elements into
  set LoopList ""
  set StartIndex 0
  if { $LMActiveLoop >= 0 } \
    {
    foreach i $LMLoops($LMActiveLoop,O) \
      { 
      set LoopList "$LoopList [expr $i+1]" 
      if { [expr $i+1] == $LMActiveFrame } { set StartIndex $i }
      }
    # -- don't allow loading outside of the loop or doubling back over same frame  
    if { $LMLoops($LMActiveLoop,F) < $GridNumber } { return }
    } \
  else \
    {
    #--- Need to get all of the frames that are not in a loop and build a list of them
    for { set i 0 } { $i < $LMLoopCount } { incr i } \
      {  
      if { $LMLoops($i,F) > 0 } \
        {
        foreach value $LMLoops($i,O) { set Marked($value) 1 }
	}
      }
    set j 0  
    for { set i 0 } { $i < $LMFrameCount } { incr i} \
      {
      # -- if the loop isn't marked, or if the active loop is ALL FRAMES
      if { ! [info exists Marked($i)] || $LMActiveLoop==-3 } \
        { 
	set LoopList "$LoopList [expr $i + 1]" 
        if { [expr $i+1] == $LMActiveFrame } { set StartIndex $j }
	incr j
	}
      }
    # -- don't allow loading outside of the loop or doubling back over same frame  
    if { [llength $LoopList] < $GridNumber } {  return }
    }
  # get position of the ActiveFrame in this list
  # Now the frame to set is an offset from there
    set Count [llength $LoopList]    
    if { ! $Count } { set SetFrame $LMActiveFrame } \
    else \
      {
      set Where [expr ($GridNumber + $StartIndex -1) % $Count]
      set SetFrame [lindex $LoopList $Where]
      }
  
  mcidas exec "SF $SetFrame"
  if { $Position==-9999 } \
    {
    mcidas exec "GRDDISP $Group/$Descriptor.$Directory $SetFrame $LoadOptions"
    }\
  else \
    {  
    mcidas exec "GRDDISP $Group/$Descriptor.$Directory $SetFrame GRID=$Position $LoadOptions"
    }
  LMRefreshFrames
  LMUpdateLoopButtons 
  }
  
#------------------------------------------------------------------------------#
# LoadPointsIntoFrame { Group Descriptor Position Options }
#------------------------------------------------------------------------------#
proc LMLoadPointsIntoFrame { Group Descriptor Position LoadOptions PointNumber} \
  {
  global LMActiveFrame LMActiveLoop
  global LMFrames LMLoops 
  global LMFrameCount
  global LMLoopCount
  
  #---- Build the list of frames to insert elements into
  set LoopList ""
  set StartIndex 0
  if { $LMActiveLoop >= 0 } \
    {
    foreach i $LMLoops($LMActiveLoop,O) \
      { 
      set LoopList "$LoopList [expr $i+1]" 
      if { [expr $i+1] == $LMActiveFrame } { set StartIndex $i }
      }
    } \
  else \
    {
    #--- Need to get all of the frames that are not in a loop and build a list of them
    for { set i 0 } { $i < $LMLoopCount } { incr i } \
      {  
      if { $LMLoops($i,F) > 0 } \
        {
        foreach value $LMLoops($i,O) { set Marked($value) 1 }
	}
      }
    set j 0  
    for { set i 0 } { $i < $LMFrameCount } { incr i} \
      {
      # -- if the loop isn't marked, or if the active loop is ALL FRAMES
      if { ! [info exists Marked($i)] || $LMActiveLoop==-3 } \
        { 
	set LoopList "$LoopList [expr $i + 1]" 
        if { [expr $i+1] == $LMActiveFrame } { set StartIndex $j }
	incr j
	}
      }
    }
  # get position of the ActiveFrame in this list
  # Now the frame to set is an offset from there
    set Count [llength $LoopList]    
    if { ! $Count } { set SetFrame $LMActiveFrame } \
    else \
      {
      set Where [expr ($PointNumber + $StartIndex -1) % $Count]
      set SetFrame [lindex $LoopList $Where]
      }
  
  mcidas exec "SF $SetFrame"
  mcidas exec "PTDISP $Group/$Descriptor.$Position $SetFrame $LoadOptions"
  LMRefreshFrames
  LMUpdateLoopButtons 
  }
  
